%%This code carries out the estimation for the bus engine replacement Monte
%Carlo. The code inputs simulated data generated by just_data.m. To
%generate all results change the variable 'sample_size' to all the sizes
%given in the tables and set the variable 'powers' to generate the
%desired basis functions. The filenames for data to be loaded and for the
%locations of the saved estimates are generated automatically from the
%choices of 'sample_size' and 'powers'.
%
%Ben Deaner 2020 (bendeaner@gmail.com)

clear

periods=10;
sample_size=100;

powers=1; %order of polynomial terms for the basis functions:
          %1 - only linear terms
          %1.5 - linear and squared terms
          %2 - linear, squared terms, and interactions
          %>=2 - all powers and interactions up to that order
          
          
if powers==1
    suffix='';
end
if powers==1.5
    suffix='_squares';
end
if powers==2
    suffix='_quad';
end
if powers>2
    suffix=num2str(powers);
end

data_path=strcat('MCdata_t=',num2str(periods),'_n=',num2str(sample_size),suffix,'.mat');
save_path_est=strcat('MCestimates_t=',num2str(periods),'_n=',num2str(sample_size),suffix,'.mat');
save_path_sd=strcat('MCstandard_errors_t=',num2str(periods),'_n=',num2str(sample_size),suffix,'.mat');

load(data_path); %load simulated data from just_data.m

draws=size(data,4); %find number of Monte Carlo draws
periods=size(data,2); %number of individual engines in each Monte Carlo sample
sample_size=size(data,1); %number of individual engines in each Monte Carlo sample

number_multi_index_states=((size(data,3)-2)/2); %number of multi-index (iid) states
delta=0.9; %discount factor

splitnum = 5; %number of subsamples to use for sample splitting

splitsize=floor(sample_size/splitnum);

if (sample_size/splitnum)-splitsize>0
    data(splitsize*splitnum+1:end,:,:,:)=[];
end

%flatten the data
Data=permute(reshape(permute(data,[2,1,3,4]),[size(data,1)*size(data,2),1,size(data,3),size(data,4)]),[1,3,4,2]);

%figure out all the split indices
I=logical(kron(eye(splitnum),ones(periods*splitsize,1)));

%% estimate parameters for each Monte Carlo draw

est=nan(2,4,draws); %we will save the coefficient estimates here
sd=nan(2,4,draws); %we will save the estimated standard errors here

%note: estimation code returns two matrices. The first 'estimates' or
%'esttmp' below, is a 2-by-4 matrix whose columns correspond to estimates
%of the two parameters. The first column has no bias correction, the final
%column is fully bias corrected. The 2nd and 3rd columns are partially
%adjusted using only \phi_2 and using \phi_2 and \phi_3 respectively.
%'standard_errors' or 'sdtmp' below is a 2-by-4 matrix whose entries are
%estimated standard errors for the corresponding elements of 'estimates'.

parfor i=1:draws

    [esttmp,sdtmp] = estimate3(Data(:,:,i),I, number_multi_index_states, delta,powers);             
 
    est(:,:,i)=esttmp;
    sd(:,:,i)=sdtmp;

end

%save all the estimates from the various bootstrap draws. We save the
%estimated coefficients and the standrd errors in separate files
 save(save_path_est,'est','-v7.3');
 save(save_path_sd,'sd','-v7.3');
 
